@@ -1,14 +1,58 @@
 """
-Default configuration.
+Set default configurations in ZEN_garden.
 
-Changes from the default values are specified in config.py (folders data/tests) and system.py (individual datasets)
+This module defines default values for all configurations in ZEN_garden. The 
+class :class:`Config` serves as a container grouping all model configurations.
+The configurations are further organized in a class structure that resembles
+that of the ZEN-garden input data. The :class:`Config` class thus links to the four 
+main configuration types (``analysis``, ``solver``, ``system``, and ``scenario``), 
+each defined using separate class. Default configurations for the ``system.json`` 
+configurations are located in the class :class:`System`. Whenever a 
+configuration consists of a dictionary, a new class is defined 
+to provide a template for the configuration and define all required
+default values.  
+
+The current structure of classes in which defaults are set is as follows:
+
+.. code-block::
+
+    Config
+    |--Analysis
+    |  |--Subsets
+    |  |--HeaderDataInputs
+    |  `--TimeSeriesAggregation
+    |
+    |--Solver
+    |--System
+    `--Scenario
+
+
+Default values are overwritten by any chances specified in the input files
+``system.json``, ``scenarios.json``, and ``config.json``.
 """
 
-from pydantic import BaseModel, ConfigDict
-from typing import Any, Optional, Union
-import importlib.metadata
+from pydantic import BaseModel,ConfigDict, create_model
+from typing import Any, Optional, Union, get_type_hints
+
+
+class Subscriptable(BaseModel):
+    """
+    Allows dictionary-like access to class attributes.
+
+    This class allows dictionary-like access to class attributes, such as
+    ``obj["key"]`` instead of ``obj.key``. Similarly, attribute values can
+    be changed in a dictionary like fashion ``obj["key"] = new_value``. Lastly,
+    attribute names and values can be called using the methods ``.keys()``,
+    ``.values()``, and ``.items()`` like in a normal dictionary. 
+
+    Inherits from:
+        :class:`BaseModel` - Class from the Pydantic package which provides
+        advanced features in input data handling and validation. 
+
+    """
+
+    model_config = ConfigDict(extra="allow")
 
-class Subscriptable(BaseModel, extra="allow"):
     def __getitem__(self, __name: str) -> Any:
         return getattr(self, __name)
 
@@ -18,35 +62,12 @@ class Subscriptable(BaseModel, extra="allow"):
     def keys(self) -> Any:
         return self.model_dump().keys()
 
-    def update(self, new_values: dict[Any, Any]) -> None:
-        for key, val in new_values.items():
-            if isinstance(val, dict):
-                getattr(self, key).update(val)
-            else:
-                setattr(self, key, val)
-
     def items(self) -> Any:
         return self.model_dump().items()
 
     def values(self) -> Any:
         return self.model_dump().values()
 
-    def __iter__(self) -> Any:
-        self.fix_keys = list(self.model_dump().keys())
-        self.i = 0
-        return self
-
-    def __next__(self) -> Any:
-        if self.i < len(self.fix_keys):
-            ans = self.fix_keys[self.i]
-            self.i += 1
-            return ans
-        else:
-            del self.i
-            del self.fix_keys
-            raise StopIteration
-
-
 class Subsets(Subscriptable):
     set_carriers: list[str] = []
     set_technologies: dict[str, list[str]] | list[str] = {
@@ -57,9 +78,22 @@ class Subsets(Subscriptable):
 
 
 class HeaderDataInputs(Subscriptable):
+    
     """
-    Header data inputs for the model
+    Maps input/output headers to internal set names used in ZEN-garden.
+
+    This class defines standard header names for the input and 
+    output files of ZEN-garden. It provides a mapping between the column headers
+    of input/output files and internal set names used in the code. For 
+    example, the class attribute "set_nodes" (default value "node") means that
+    any input csv file with column header "node" will be interpreted as 
+    containing elements of the internal set "set_nodes". 
+
+    Inherits from:
+        :class:`Subscriptable` - Provides dictionary-like access to attributes 
+        and allows input data handling via Pydantic's BaseModel
     """
+
     set_nodes: str = "node"
     set_edges: str = "edge"
     set_location: str = "location"
@@ -82,12 +116,14 @@ class HeaderDataInputs(Subscriptable):
     set_technologies_existing: str = "technology_existing"
     set_capacity_types: str = "capacity_type"
 
+
 class System(Subscriptable):
     """
     Class which contains the system configuration. This defines for example the set of carriers, technologies, etc.
     """
     set_carriers: list[str] = []
     set_capacity_types: list[str] = ["power", "energy"]
+    set_technologies: list[str] = []
     set_conversion_technologies: list[str] = []
     set_storage_technologies: list[str] = []
     set_retrofitting_technologies: list[str] = []
@@ -97,6 +133,7 @@ class System(Subscriptable):
     set_transport_technologies_loss_exponential: list[str] = []
     double_capex_transport: bool = False
     set_nodes: list[str] = []
+    coords: dict[str, dict[str, float]] = {}
     exclude_parameters_from_TSA: bool = True
     conduct_scenario_analysis: bool = False
     run_default_scenario: bool = True
@@ -113,24 +150,29 @@ class System(Subscriptable):
     years_in_rolling_horizon: int = 1
     years_in_decision_horizon: int = 1
     use_capacities_existing: bool = True
+    allow_investment: bool = True
+    storage_charge_discharge_binary: bool = False
+    include_operation_only_phase: bool = False
+
 
-class SolverOptions(Subscriptable):
-    pass
 
 class Solver(Subscriptable):
     """
     Class which contains the solver configuration. This defines for example the solver options, scaling, etc.
     """
     name: str = "highs"
-    solver_options: SolverOptions = SolverOptions()
+    solver_options: dict = {}
     check_unit_consistency: bool = True
     solver_dir: str = ".//outputs//solver_files"
     keep_files: bool = False
     io_api: str = "lp"
     save_duals: bool = False
     save_parameters: bool = True
-    selected_saved_parameters: list = [] # if empty, all parameters are saved
-    selected_saved_variables: list = [] # if empty, all variables are saved
+    selected_saved_parameters: list = []  # if empty, all parameters are saved
+    selected_saved_variables: list = []  # if empty, all variables are saved
+    selected_saved_variables_operation: list = []  # if empty, all variables are saved
+    selected_saved_duals: list = []  # if empty, all duals are saved (if save_duals is True)
+    selected_saved_duals_operation: list = []  # if empty, all duals are saved (if save_duals is True)
     linear_regression_check: dict[str, float] = {
         "eps_intercept": 0.1,
         "epsRvalue": 1 - (1e-5),
@@ -143,22 +185,22 @@ class Solver(Subscriptable):
     run_diagnostics: bool = False
     use_scaling: bool = True
     scaling_include_rhs: bool = True
-    scaling_algorithm: Union[list[str],str] = ["geom","geom","geom"]
+    scaling_algorithm: Union[list[str], str] = ["geom", "geom", "geom"]
 
 
 class TimeSeriesAggregation(Subscriptable):
     """
     Class which contains the time series aggregation configuration. This defines for example the clustering method, etc.
     """
-    slv: Solver = Solver()
     clusterMethod: str = "hierarchical"
-    solver: str = slv.name
-    hoursPerPeriod: int = 1 # keep this at 1
+    solver: str = "highs"
+    hoursPerPeriod: int = 1  # keep this at 1
     extremePeriodMethod: Optional[str] = "None"
     rescaleClusterPeriods: bool = False
     representationMethod: str = "meanRepresentation"
     resolution: int = 1
 
+
 class Analysis(Subscriptable):
     """
     Class which contains the analysis configuration. This defines for example the objective function, output settings, etc.
@@ -175,9 +217,11 @@ class Analysis(Subscriptable):
     earliest_year_of_data: int = 1900
     zen_garden_version: str = None
 
+
+
 class Config(Subscriptable):
     """
-    Class which contains the configuration of the model. This includes the configuratins of the system, solver, and analysis as well as the dictionary of scenarios.
+    Class which contains the configuration of the model. This includes the configurations of the system, solver, and analysis as well as the dictionary of scenarios.
     """
     analysis: Analysis = Analysis()
     solver: Solver = Solver()
