@@ -1,8 +1,8 @@
 """
 This module contains the Results class, which is used to extract and process the results of a model run.
 """
-import pandas as pd
-from typing import Optional, Any, Literal
+from pandas import Series
+
 from zen_garden.postprocess.results.solution_loader import (
     SolutionLoader,
     Scenario,
@@ -10,14 +10,19 @@ from zen_garden.postprocess.results.solution_loader import (
     TimestepType,
     ComponentType,
 )
-from functools import cache
 from zen_garden.model.default_config import Config, Analysis, Solver, System
+import pandas as pd
+from typing import Optional, Any, Literal, Union
+from zen_garden.utils import reformat_slicing_index
+from functools import cache
 import importlib
 import os
 import logging
 import json
 from pathlib import Path
 
+NestedTuple = tuple[list[str]] | tuple[str]
+NestedDict = dict[str, str | list[str]]
 
 class Results:
     """
@@ -29,6 +34,8 @@ class Results:
 
         :param path: Path to the results folder
         """
+        assert os.path.exists(path), f"The output folder {Path(path).absolute()} does not exist."
+        assert len(os.listdir(path)) > 0, f"The output folder {Path(path).absolute()} is empty."
         self.solution_loader = SolutionLoader(path)
         self.has_scenarios = len(self.solution_loader.scenarios) > 1
         first_scenario = next(iter(self.solution_loader.scenarios.values()))
@@ -44,37 +51,54 @@ class Results:
         component_name: str,
         scenario_name: Optional[str] = None,
         data_type: Literal["dataframe", "units"] = "dataframe",
-    ) -> Optional[dict[str, "pd.DataFrame | pd.Series[Any]"]]:
+        index: Optional[Union[NestedTuple, NestedDict, list[str], str, float, int]] = None,
+    ) -> Optional[Union[dict[str, "pd.DataFrame | pd.Series[Any]"],pd.Series]]:
         """
+        Returns the raw results without any further processing.
         Transforms a parameter or variable dataframe (compressed) string into an actual pandas dataframe
 
         :component_name string: The string to decode
         :scenario_name: Which scenario to take. If none is specified, all are returned.
+        :data_type: The type of data to extract. Either 'dataframe' or 'units'
+        :index: slicing index of the resulting dataframe
         :return: The corresponding dataframe
         """
+
         scenario_names = (
-            self.solution_loader.scenarios.keys()
+            list(self.solution_loader.scenarios.keys())
             if scenario_name is None
             else [scenario_name]
         )
 
-        if component_name not in self.solution_loader.components:
-            logging.warning(f"Component {component_name} not found. If you expected this component to be present, the solution is probably empty and therefore skipped.")
-            return {s:pd.Series() for s in scenario_names}
-
-        component = self.solution_loader.components[component_name]
-
-        if data_type == "units" and not component.has_units:
-            return None
-
-        ans = {}
-
-        for scenario_name in scenario_names:
+        if len(scenario_names) == 1:
+            scenario_name = scenario_names[0]
             scenario = self.solution_loader.scenarios[scenario_name]
-            ans[scenario_name] = self.solution_loader.get_component_data(
-                scenario, component, data_type=data_type
+            if component_name not in scenario.components:
+                logging.warning(f"Component {component_name} not found. If you expected this component to be present, the solution is probably empty and therefore skipped.")
+                return pd.Series()
+            component = scenario.components[component_name]
+            if data_type == "units" and not component.has_units:
+                return None
+            idx = reformat_slicing_index(index,component)
+            ans = self.solution_loader.get_component_data(
+                scenario, component, data_type=data_type, index=idx
             )
-
+        else:
+            ans = {}
+            for scenario_name in scenario_names:
+                scenario = self.solution_loader.scenarios[scenario_name]
+                if component_name not in scenario.components:
+                    continue
+                component = scenario.components[component_name]
+                if data_type == "units" and not component.has_units:
+                    return None
+                idx = reformat_slicing_index(index, component)
+                ans[scenario_name] = self.solution_loader.get_component_data(
+                    scenario, component, data_type=data_type, index=idx
+                )
+            if len(ans) == 0:
+                logging.warning(f"Component {component_name} not found. If you expected this component to be present, the solution is probably empty and therefore skipped.")
+                return {}
         return ans
 
     def get_full_ts_per_scenario(
@@ -83,40 +107,48 @@ class Results:
         component: Component,
         year: Optional[int] = None,
         discount_to_first_step: bool = True,
-        element_name: Optional[str] = None,
         keep_raw: Optional[bool] = False,
-    ) -> "pd.Series[Any]":
-        """Calculates the full timeseries for a given element per scenario
+        index: tuple[str] = None
+    ) -> "pd.DataFrame":
+        """Calculates the full timeseries per scenario
 
         :param scenario: The scenario for with the component should be extracted (only if needed)
         :param component: Component for the Series
         :param discount_to_first_step: apply annuity to first year of interval or entire interval
         :param year: year of which full time series is selected
-        :param element_name: Filter results by a given element
         :param keep_raw: Keep the raw values of the rolling horizon optimization
-        :return: Full timeseries of the element
+        :param index: slicing index of the resulting dataframe
+        :return: Full timeseries
         """
         assert component.timestep_type is not None, "Component has no timestep type."
+        
+        if index is None:
+            index = tuple()
 
-        series = self.solution_loader.get_component_data(
-            scenario, component, keep_raw=keep_raw
+        sequence_timesteps = self.solution_loader.get_sequence_time_steps(
+            scenario, component.timestep_type
         )
-        if element_name is not None and element_name in series.index.get_level_values(0):
-            series = series.loc[element_name]
-
         if year is None:
             years = [i for i in range(0, scenario.system.optimized_years)]
         else:
+            year = scenario.convert_year2ts(year)
             years = [year]
 
+        # slice index with time steps of year
+        select_year_time_steps = False
+        if component.timestep_type is TimestepType.operational or component.timestep_type is TimestepType.storage:
+            if not any(str(component.timestep_type.value) in i for i in index):
+                time_steps = self.solution_loader.get_timesteps_of_years(scenario, component.timestep_type,tuple(years)).values
+                index = index + (f"{component.timestep_type.value} in [{', '.join(time_steps.astype(str))}]",)
+                select_year_time_steps = True
+        series = self.solution_loader.get_component_data(
+            scenario, component, keep_raw=keep_raw, index=index
+        )
         if isinstance(series.index, pd.MultiIndex):
             series = series.unstack(component.timestep_name)
 
         if component.timestep_type is TimestepType.yearly:
-            if component.component_type not in [
-                ComponentType.parameter,
-                ComponentType.variable,
-            ]:
+            if component.component_type is ComponentType.dual:
                 annuity = self._get_annuity(scenario, discount_to_first_step)
                 ans = series / annuity
             else:
@@ -126,13 +158,9 @@ class Results:
                 ans = ans[years]
             except KeyError:
                 pass
-
+            ans = scenario.convert_ts2year(ans)
             return ans
 
-        sequence_timesteps = self.solution_loader.get_sequence_time_steps(
-            scenario, component.timestep_type
-        )
-
         if (
             component.component_type is ComponentType.dual
             and component.timestep_type is not None
@@ -151,14 +179,19 @@ class Results:
                 series[time_steps_year] = series[time_steps_year] / annuity[year_temp]
         try:
             if component.timestep_type is TimestepType.operational:
+                if select_year_time_steps:
+                    sequence_timesteps = sequence_timesteps[sequence_timesteps.isin(time_steps)]
                 output_df = series[sequence_timesteps]
             elif component.timestep_type is TimestepType.storage:
                 # for storage components, the last timestep is the final state, linear interpolation is used
                 last_occurrences = sequence_timesteps.groupby(sequence_timesteps).apply(lambda x: x.index[-1])
                 first_occurrences = sequence_timesteps.groupby(sequence_timesteps).apply(lambda x: x.index[0])
-                output_df = pd.DataFrame(columns=sequence_timesteps.index,index=series.index,dtype=float)
-                output_df[last_occurrences.values] = series[last_occurrences.index]
+                last_occurrences = last_occurrences[last_occurrences.index.intersection(series.columns)]
+                output_df = series[last_occurrences.index].rename(last_occurrences,axis=1)
+                # fill missing ts with nan
+                output_df = output_df.reindex(columns=sequence_timesteps.index)
                 time_steps_start_end = self.solution_loader.get_time_steps_storage_level_startend_year(scenario)
+                time_steps_start_end = {k:v for k,v in time_steps_start_end.items() if k in first_occurrences and v in last_occurrences}
                 for tstart,tend in time_steps_start_end.items():
                     tstart_reconstructed = first_occurrences[tstart]
                     first_valid_timestep = output_df.loc[:,tstart_reconstructed:].T.first_valid_index()
@@ -168,6 +201,9 @@ class Results:
                     df_temp = df_temp.interpolate(method='index',axis=1)
                     output_df.loc[:,first_occurrences[tstart]:last_occurrences[tstart]] = df_temp.loc[:,tstart_reconstructed:first_valid_timestep]
                 output_df = output_df.interpolate(method='index',axis=1)
+                if select_year_time_steps:
+                    sequence_timesteps = sequence_timesteps[sequence_timesteps.isin(time_steps)]
+                output_df = output_df[sequence_timesteps.index]
             else:
                 raise ValueError(f"Invalid timestep type {component.timestep_type} for component {component}")
         except KeyError:
@@ -175,98 +211,92 @@ class Results:
 
         output_df = output_df.T.reset_index(drop=True).T
 
-        if year is not None:
-            _total_hours_per_year = scenario.system.unaggregated_time_steps_per_year
-
-            hours_of_year = list(
-                range(year * _total_hours_per_year, (year + 1) * _total_hours_per_year)
-            )
-            if output_df.empty:
-                return pd.DataFrame(index=[],columns=hours_of_year)
-            output_df = output_df[hours_of_year]
         return output_df
 
     def get_full_ts(
         self,
         component_name: str,
         scenario_name: Optional[str] = None,
-        discount_to_first_step: bool = True,
+        discount_to_first_step: Optional[bool] = True,
         year: Optional[int] = None,
-        element_name: Optional[str] = None,
         keep_raw: Optional[bool] = False,
+        index: Optional[Union[NestedTuple, NestedDict, list[str], str, float, int]] = None,
     ) -> "pd.DataFrame | pd.Series[Any]":
-        """Calculates the full timeseries for a given element
+        """Calculates the full timeseries
 
         :param component_name: Name of the component
         :param scenario_name: The scenario for with the component should be extracted (only if needed)
         :param discount_to_first_step: apply annuity to first year of interval or entire interval
         :param year: year of which full time series is selected
-        :param element_name: Filter results by a given element
         :param keep_raw: Keep the raw values of the rolling horizon optimization
-        :return: Full timeseries of the element
+        :param index: slicing index of the resulting dataframe
+        :return: Full timeseries
         """
         if scenario_name is None:
             scenario_names = list(self.solution_loader.scenarios)
         else:
             scenario_names = [scenario_name]
 
-        if component_name not in self.solution_loader.components:
-            logging.warning(f"Component {component_name} not found. If you expected this component to be present, the solution is probably empty and therefore skipped.")
-            return pd.Series()
-
-        component = self.solution_loader.components[component_name]
-
         scenarios_dict: dict[str, "pd.DataFrame | pd.Series[Any]"] = {}
 
         for scenario_name in scenario_names:
             scenario = self.solution_loader.scenarios[scenario_name]
-
+            if component_name not in scenario.components:
+                continue
+            component = scenario.components[component_name]
+            idx = reformat_slicing_index(index,component)
             scenarios_dict[scenario_name] = self.get_full_ts_per_scenario(
                 scenario,
                 component,
                 discount_to_first_step=discount_to_first_step,
                 year=year,
-                element_name=element_name,
                 keep_raw=keep_raw,
+                index=idx
             )
+        if len(scenarios_dict) == 0:
+            logging.warning(f"Component {component_name} not found. If you expected this component to be present, the solution is probably empty and therefore skipped.")
+            return pd.Series()
 
-        return self._concat_scenarios_dict(scenarios_dict)
+        return self._concat_scenarios_dict(scenarios_dict,scenario_names)
 
-    @cache
     def get_total_per_scenario(
         self,
         scenario: Scenario,
         component: Component,
-        element_name: Optional[str] = None,
         year: Optional[int] = None,
         keep_raw: Optional[bool] = False,
+        index: tuple[str] = None
     ) -> "pd.DataFrame | pd.Series[Any]":
         """
         Calculates the total values of a component for a specific scenario.
 
         :param scenario: Scenario
         :param component: Component
-        :param element_name: Filter the results by a given element
         :param year: Filter the results by a given year
         :param keep_raw: Keep the raw values of the rolling horizon optimization
+        :param index: slicing index of the resulting dataframe
         :return: Total values of the component
         """
-        series = self.solution_loader.get_component_data(scenario, component, keep_raw)
+        if index is None:
+            index = tuple()
+        series = self.solution_loader.get_component_data(scenario, component, keep_raw, index = index)
 
         if year is None:
             years = [i for i in range(0, scenario.system.optimized_years)]
         else:
+            year = scenario.convert_year2ts(year)
             years = [year]
 
-        if element_name is not None:
-            series = series.loc[element_name]
-
         if component.timestep_type is None or type(series.index) is not pd.MultiIndex:
+            if component.timestep_type is TimestepType.yearly:
+                series = scenario.convert_ts2year(series)
             return series
 
         if component.timestep_type is TimestepType.yearly:
             ans = series.unstack(component.timestep_name)
-            return ans[years]
+            ans = ans[years]
+            ans = scenario.convert_ts2year(ans)
+            return ans
 
         timestep_duration = self.solution_loader.get_timestep_duration(
             scenario, component
@@ -289,26 +319,25 @@ class Results:
             ans = ans.reorder_levels(
                 [i for i in ans.index.names if i != "mf"] + ["mf"]
             ).sort_index(axis=0)
-
+        ans = scenario.convert_ts2year(ans)
         return ans
 
-    @cache
     def get_total(
         self,
         component_name: str,
-        element_name: Optional[str] = None,
         year: Optional[int] = None,
         scenario_name: Optional[str] = None,
         keep_raw: Optional[bool] = False,
+        index: Optional[Union[NestedTuple, NestedDict, list[str], str, float, int]] = None,
     ) -> "pd.DataFrame | pd.Series[Any]":
         """
         Calculates the total values of a component for a all scenarios.
 
         :param component_name: Name of the component
-        :param element_name: Filter the results by a given element
         :param year: Filter the results by a given year
         :param scenario_name: Filter the results by a given scenario
         :param keep_raw: Keep the raw values of the rolling horizon optimization
+        :param index: slicing index of the resulting dataframe
         :return: Total values of the component
         """
         if scenario_name is None:
@@ -316,18 +345,16 @@ class Results:
         else:
             scenario_names = [scenario_name]
 
-        if component_name not in self.solution_loader.components:
-            logging.warning(f"Component {component_name} not found. If you expected this component to be present, the solution is probably empty and therefore skipped.")
-            return pd.Series()
-
-        component = self.solution_loader.components[component_name]
-
         scenarios_dict: dict[str, "pd.DataFrame | pd.Series[Any]"] = {}
 
         for scenario_name in scenario_names:
             scenario = self.solution_loader.scenarios[scenario_name]
+            if component_name not in scenario.components:
+                continue
+            component = scenario.components[component_name]
+            idx = reformat_slicing_index(index, component)
             current_total = self.get_total_per_scenario(
-                scenario, component, element_name, year, keep_raw
+                scenario, component, year, keep_raw, index = idx
             )
 
             if type(current_total) is pd.Series:
@@ -335,10 +362,14 @@ class Results:
 
             scenarios_dict[scenario_name] = current_total
 
-        return self._concat_scenarios_dict(scenarios_dict)
+        if len(scenarios_dict) == 0:
+            logging.warning(f"Component {component_name} not found. If you expected this component to be present, the solution is probably empty and therefore skipped.")
+            return pd.Series()
+
+        return self._concat_scenarios_dict(scenarios_dict,scenario_names)
 
     def _concat_scenarios_dict(
-        self, scenarios_dict: dict[str, "pd.DataFrame | pd.Series[Any]"]
+        self, scenarios_dict: dict[str, "pd.DataFrame | pd.Series[Any]"],scenario_names: list[str]
     ) -> pd.DataFrame:
         """
         Concatenates a dict of the form str: Data to one dataframe.
@@ -346,12 +377,10 @@ class Results:
         :param scenarios_dict: Dict containing the scenario names as key and the values as values.
         :return: Concatenated dataframe
         """
-        scenario_names = list(scenarios_dict.keys())
-
         if len(scenario_names) == 1:
             ans = scenarios_dict[scenario_names[0]]
             return ans
-
+        scenario_names = list(scenarios_dict.keys())
         if isinstance(scenarios_dict[scenario_names[0]], pd.Series):
             total_value = pd.concat(
                 scenarios_dict, keys=scenarios_dict.keys(), axis=1
@@ -375,7 +404,7 @@ class Results:
         :return: annuity of the duals
         """
         system = scenario.system
-        discount_rate_component = self.solution_loader.components["discount_rate"]
+        discount_rate_component = scenario.components["discount_rate"]
         # calculate annuity
         discount_rate = self.solution_loader.get_component_data(
             scenario, discount_rate_component
@@ -416,37 +445,32 @@ class Results:
         self,
         component_name: str,
         scenario_name: Optional[str] = None,
-        element_name: Optional[str] = None,
         year: Optional[int] = None,
         discount_to_first_step: bool = True,
         keep_raw: Optional[bool] = False,
+        index: Optional[Union[NestedTuple, NestedDict, list[str], str, float, int]] = None,
     ) -> Optional["pd.DataFrame | pd.Series[Any]"]:
         """extracts the dual variables of a component
 
-        :param component: Name of dal
+        :param component_name: Name of dual
         :param scenario_name: Scenario Name
-        :param element_name: Name of Element
         :param year: Year
         :param discount_to_first_step: apply annuity to first year of interval or entire interval
         :param keep_raw: Keep the raw values of the rolling horizon optimization
+        :param index: slicing index of the resulting dataframe
         :return: Duals of the component
         """
         if not self.get_solver(scenario_name=scenario_name).save_duals:
             logging.warning("Duals are not calculated. Skip.")
             return None
 
-        component = self.solution_loader.components[component_name]
-        assert (
-            component.component_type is ComponentType.dual
-        ), f"Given component {component} is not of type Dual."
-
         duals = self.get_full_ts(
             component_name=component_name,
             scenario_name=scenario_name,
-            element_name=element_name,
             year=year,
             discount_to_first_step=discount_to_first_step,
             keep_raw=keep_raw,
+            index=index,
         )
         return duals
 
@@ -454,25 +478,27 @@ class Results:
         self,
         component_name: str,
         scenario_name: Optional[str] = None,
+        index: Optional[Union[NestedTuple, NestedDict, list[str], str, float, int]] = None,
         droplevel: bool = True,
-        is_total: bool = True,
-    ) -> Optional[dict[str, "pd.DataFrame | pd.Series[Any]"]]:
+        convert_to_yearly_unit: bool = False,
+    ) -> None | Series | str:
         """
         Extracts the unit of a given Component. If no scenario is given, a random one is taken.
 
         :param component_name: Name of the component
         :param scenario_name: Name of the scenario
+        :param index: slicing index of the resulting dataframe
         :param droplevel: Drop the location and time levels of the multiindex
+        :param convert_to_yearly_unit: If True, the unit is converted to a yearly unit, i.e., for components with an operational time step type, the unit is multiplied by hours.
         :return: The corresponding unit
         """
         if scenario_name is None:
             scenario_name = next(iter(self.solution_loader.scenarios.keys()))
-        res = self.get_df(
-            component_name, scenario_name=scenario_name, data_type="units"
+        units = self.get_df(
+            component_name, scenario_name=scenario_name, data_type="units", index=index
         )
-        if res is None:
+        if units is None:
             return None
-        units = res[scenario_name]
         if droplevel:
             # TODO make more flexible
             loc_idx = ["node", "location", "edge", "set_location", "set_nodes"]
@@ -489,23 +515,45 @@ class Results:
                 units.index = units.index.droplevel(drop_idx.to_list())
                 units = units[~units.index.duplicated()]
         # convert to pint units
-        for i in units.index:
-            try:
-                u = units[i]
-                u = self.ureg.parse_expression(u)
-                if is_total and self.solution_loader.components[component_name].timestep_type is TimestepType.operational:
-                    u = u * self.ureg.h
-                units[i] = f"{u.u:~D}"
-            # if the unit is not in the pint registry, change the string manually (normally, when the unit_definition.txt is not saved)
-            except Exception:
-                if is_total and self.solution_loader.components[component_name].timestep_type is TimestepType.operational:
-                    if units[i].endswith(" / hour"):
-                        units[i] = units[i].replace(" / hour", "")
-                    else:
-                        units[i] = f"{units[i]} * hour"
+        if isinstance(units, pd.Series):
+            for i in units.index:
+                units[i] = self._convert_to_pint_units(units[i], convert_to_yearly_unit, component_name)
+        elif isinstance(units, str):
+            units = self._convert_to_pint_units(units, convert_to_yearly_unit, component_name)
+        else:
+            raise TypeError(f"Invalid units type: {type(units)}")
 
         return units
 
+    def _convert_to_pint_units(self,u: str,convert_to_yearly_unit: bool, component_name: str) -> str:
+        """
+        Converts a string to a pint unit.
+        """
+        component = None
+        for s in self.solution_loader.scenarios:
+            if component_name in self.solution_loader.scenarios[s].components:
+                component = self.solution_loader.scenarios[s].components[component_name]
+                break
+        if component is None:
+            return u
+        timestep_type = component.timestep_type
+
+        try:
+            u = self.ureg.parse_expression(u)
+            if convert_to_yearly_unit and timestep_type is TimestepType.operational:
+                u = u * self.ureg.h
+            u_return = f"{u.u:~D}"
+        # if the unit is not in the pint registry, change the string manually (normally when the unit_definition.txt is not saved)
+        except Exception:
+            if convert_to_yearly_unit and timestep_type is TimestepType.operational:
+                if u.endswith(" / hour"):
+                    u_return = u.replace(" / hour", "")
+                else:
+                    u_return = f"{u} * hour"
+            else:
+                u_return = u
+        return u_return
+
     def get_system(self, scenario_name: Optional[str] = None) -> System:
         """
         Extracts the System config of a given Scenario. If no scenario is given, a random one is taken.
@@ -539,14 +587,22 @@ class Results:
             scenario_name = next(iter(self.solution_loader.scenarios.keys()))
         return self.solution_loader.scenarios[scenario_name].solver
 
-    def get_doc(self, component: str) -> str:
+    def get_doc(self, component_name: str) -> str:
         """
         Extracts the documentation of a given Component.
 
-        :param component: Name of the component
+        :param component_name: Name of the component
         :return: The corresponding documentation
         """
-        return self.solution_loader.components[component].doc
+        component = None
+        for scenario in self.solution_loader.scenarios.values():
+            if component_name in scenario.components:
+                component = scenario.components[component_name]
+                break
+        if component is None:
+            logging.warning(f"Component {component_name} not found and the documentation cannot be returned.")
+            return ""
+        return component.doc
 
     def get_years(self, scenario_name: Optional[str] = None) -> list[int]:
         """
@@ -573,6 +629,26 @@ class Results:
         scenario = self.solution_loader.scenarios[scenario_name]
         return scenario.system.use_rolling_horizon
 
+    def get_coords(self, scenario_name: Optional[str] = None) -> Optional[pd.DataFrame]:
+        """
+        Extracts the coordinates of the nodes of a given Scenario. If no scenario is given, a random one is taken.
+
+        :param scenario_name: Name of the scenario
+        :return: The corresponding coordinates
+        """
+        if scenario_name is None:
+            scenario_name = next(iter(self.solution_loader.scenarios.keys()))
+        system = self.get_system(scenario_name)
+        if hasattr(system,"coords"):
+            coords = pd.DataFrame(system.coords).T
+            if coords.empty:
+                print(f"Coordinates of nodes are not saved for version {self.get_analysis().zen_garden_version}.")
+                return None
+            return pd.DataFrame(system.coords).T
+        else:
+            print(f"Coordinates of nodes are not saved for version {self.get_analysis().zen_garden_version}.")
+            return None
+
     def calculate_connected_edges(
         self, node: str, direction: str, set_nodes_on_edges: dict[str, str]
     ):
@@ -615,7 +691,7 @@ class Results:
         if "carrier" not in dataframe.index.names:
             reference_carriers = self.get_df(
                 "set_reference_carriers", scenario_name=scenario_name
-            )[scenario_name]
+            )
             data_extracted = pd.DataFrame()
             for tech in dataframe.index.get_level_values("technology"):
                 if reference_carriers[tech] == carrier:
@@ -627,7 +703,7 @@ class Results:
 
         # check if desired carrier isn't contained in data (otherwise .loc raises an error)
         if carrier not in dataframe.index.get_level_values("carrier"):
-            return None
+            return pd.DataFrame()
 
         return dataframe.query(f"carrier == '{carrier}'")
 
@@ -638,14 +714,11 @@ class Results:
 
         :param data: pd.DataFrame containing data to extract
         :param node: node of interest
-        :param carrier: carrier of interest
         :param direction: flow direction with respect to node
         :param scenario: scenario of interest
         :return: pd.DataFrame containing carrier_flow data desired
         """
-        set_nodes_on_edges = self.get_df("set_nodes_on_edges", scenario_name=scenario)[
-            scenario
-        ]
+        set_nodes_on_edges = self.get_df("set_nodes_on_edges", scenario_name=scenario)
         set_nodes_on_edges = {
             edge: set_nodes_on_edges[edge].split(",")
             for edge in set_nodes_on_edges.index
@@ -736,7 +809,15 @@ class Results:
         :return: List of component names
         """
         assert component_type in ComponentType.get_component_type_names(), f"Invalid component type: {component_type}. Valid types are: {ComponentType.get_component_type_names()}"
-        return [component for component in self.solution_loader.components if self.solution_loader.components[component].component_type.name == component_type]
+        list_names = []
+        for scenario in self.solution_loader.scenarios:
+            for component in self.solution_loader.scenarios[scenario].components:
+                component_name = self.solution_loader.scenarios[scenario].components[component].name
+                component_type_specific = self.solution_loader.scenarios[scenario].components[component].component_type.name
+                if component_name not in list_names and component_type_specific == component_type:
+                    list_names.append(component)
+        return list_names
+
 
 if __name__ == "__main__":
     try:
